@@ -46,6 +46,30 @@ def parse_args():
             'Uses sample names as a prefix for BCR data if metadata CSV ' +
             'file absent and more than one sample to process. ' +
             'Defaults to "_".'))
+    parser.add_argument(
+        '--flavour',
+        type=str,
+        default="strict",
+        help=
+        ('The "flavour" for running igblastn reannotation. Accepts either ' +
+         '"strict" or "original". strict will enforce evalue and penalty cutoffs.'
+         ))
+    parser.add_argument(
+        '--skip_format_header',
+        action='store_true',
+        help=('If passed, skips formatting of contig headers.'))
+    parser.add_argument(
+        '--filter_to_high_confidence',
+        action='store_true',
+        help=(
+            'If passed, limits the contig space to ones that are set to ' +
+            '"True" in the high_confidence column of the contig annotation.'))
+    parser.add_argument(
+        '--skip_reassign_dj',
+        action='store_false',
+        help=(
+            'If passed, skips reassigning d/j calls with blastn when flavour=strict.'
+        ))
     parser.add_argument(
         '--keep_trailing_hyphen_number',
         action='store_false',
@@ -73,17 +97,32 @@ def main():
     args = parse_args()
     start = logg.info('\nBeginning preprocessing\n')
 
+    if args.keep_trailing_hyphen_number:
+        keep_trailing_hyphen_number_log = False
+    else:
+        keep_trailing_hyphen_number_log = True
+
+    if args.skip_reassign_dj:
+        skip_reassign_dj_log = False
+    else:
+        skip_reassign_dj_log = True
+
     logg.info(
         'command line parameters:\n',
-        deep=(f'\n'
-              f'--------------------------------------------------------------\n'
-              f'    --meta = {args.meta}\n'
-              f'    --chain = {args.chain}\n'
-              f'    --file_prefix = {args.file_prefix}\n'
-              f'    --sep = {args.sep}\n'
-              f'    --keep_trailing_hyphen_number = {args.keep_trailing_hyphen_number}\n'
-              f'    --clean_output = {args.clean_output}\n'
-              f'--------------------------------------------------------------\n'),
+        deep=
+        (f'\n'
+         f'--------------------------------------------------------------\n'
+         f'    --meta = {args.meta}\n'
+         f'    --chain = {args.chain}\n'
+         f'    --file_prefix = {args.file_prefix}\n'
+         f'    --sep = {args.sep}\n'
+         f'    --flavour = {args.flavour}\n'
+         f'    --skip_format_header = {args.skip_format_header}\n'
+         f'    --filter_to_high_confidence = {args.filter_to_high_confidence}\n'
+         f'    --keep_trailing_hyphen_number = {keep_trailing_hyphen_number_log}\n'
+         f'    --skip_reassign_dj = {skip_reassign_dj_log}\n'
+         f'    --clean_output = {args.clean_output}\n'
+         f'--------------------------------------------------------------\n'),
     )
 
     # set up a sample list
@@ -107,46 +146,61 @@ def main():
 
     # STEP ONE - ddl.pp.format_fastas()
     # do we have a prefix/suffix?
-    if 'prefix' in meta.columns:
-        # process with prefix
-        vals = list(meta['prefix'].values)
-        ddl.pp.format_fastas(
-            samples,
-            prefix=vals,
-            sep=args.sep,
-            remove_trailing_hyphen_number=args.keep_trailing_hyphen_number,
-            filename_prefix=filename_prefixes)
-    elif 'suffix' in meta.columns:
-        # process with suffix
-        vals = list(meta['suffix'].values)
-        ddl.pp.format_fastas(
-            samples,
-            suffix=vals,
-            sep=args.sep,
-            remove_trailing_hyphen_number=args.keep_trailing_hyphen_number,
-            filename_prefix=filename_prefixes)
-    else:
-        # neither. tag with the sample names as default, if more than one
-        # sample and the data is IG
-        if (len(samples) > 1) and (args.chain == 'ig'):
+    if not args.skip_format_header:
+        if 'prefix' in meta.columns:
+            # process with prefix
+            vals = list(meta['prefix'].values)
             ddl.pp.format_fastas(
                 samples,
-                prefix=samples,
+                prefix=vals,
                 sep=args.sep,
+                high_confidence_filtering=args.filter_to_high_confidence,
+                remove_trailing_hyphen_number=args.keep_trailing_hyphen_number,
+                filename_prefix=filename_prefixes)
+        elif 'suffix' in meta.columns:
+            # process with suffix
+            vals = list(meta['suffix'].values)
+            ddl.pp.format_fastas(
+                samples,
+                suffix=vals,
+                sep=args.sep,
+                high_confidence_filtering=args.filter_to_high_confidence,
                 remove_trailing_hyphen_number=args.keep_trailing_hyphen_number,
                 filename_prefix=filename_prefixes)
         else:
-            # no need to tag as it's a single sample.
-            ddl.pp.format_fastas(
-                samples,
-                remove_trailing_hyphen_number=args.keep_trailing_hyphen_number,
-                filename_prefix=filename_prefixes)
+            # neither. tag with the sample names as default, if more than one
+            # sample and the data is IG
+            if (len(samples) > 1) and (args.chain == 'ig'):
+                ddl.pp.format_fastas(
+                    samples,
+                    prefix=samples,
+                    sep=args.sep,
+                    high_confidence_filtering=args.filter_to_high_confidence,
+                    remove_trailing_hyphen_number=args.
+                    keep_trailing_hyphen_number,
+                    filename_prefix=filename_prefixes)
+            else:
+                # no need to tag as it's a single sample.
+                ddl.pp.format_fastas(
+                    samples,
+                    high_confidence_filtering=args.filter_to_high_confidence,
+                    remove_trailing_hyphen_number=args.
+                    keep_trailing_hyphen_number,
+                    filename_prefix=filename_prefixes)
+    else:
+        ddl.pp.format_fastas(
+            samples,
+            high_confidence_filtering=args.filter_to_high_confidence,
+            remove_trailing_hyphen_number=False,
+            filename_prefix=filename_prefixes)
 
     # STEP TWO - ddl.pp.reannotate_genes()
     # no tricks here
     ddl.pp.reannotate_genes(samples,
                             loci=args.chain,
-                            filename_prefix=filename_prefixes)
+                            filename_prefix=filename_prefixes,
+                            flavour=args.flavour,
+                            reassign_dj=args.skip_reassign_dj)
 
     # IG requires further preprocessing, TR is done now
     if args.chain == 'ig':
@@ -192,10 +246,10 @@ def main():
         for s in samples:
             ddl.pp.quantify_mutations(s + '/dandelion/' +
                                       str(args.file_prefix) +
-                                      '_contig_igblast_db-pass_genotyped.tsv')
+                                      '_contig_dandelion.tsv')
             ddl.pp.quantify_mutations(s + '/dandelion/' +
                                       str(args.file_prefix) +
-                                      '_contig_igblast_db-pass_genotyped.tsv',
+                                      '_contig_dandelion.tsv',
                                       frequency=True)
 
     # at this stage it's safe to remove the per-sample dandelion/tmp folder if
