@@ -17,7 +17,7 @@ def test_container():
     os.system(
         "cd /tests; python /share/dandelion_preprocess.py --meta test.csv;")
     dat = pd.read_csv(
-        '/tests/sample_test_10x/dandelion/filtered_contig_igblast_db-pass_genotyped.tsv',
+        '/tests/sample_test_10x/dandelion/filtered_contig_dandelion.tsv',
         sep='\t')
     assert not dat['c_call'].empty
     assert not dat['v_call_genotyped'].empty
