@@ -2,7 +2,7 @@
 # @Author: Kelvin
 # @Date:   2021-02-11 12:22:40
 # @Last Modified by:   Kelvin
-# @Last Modified time: 2021-08-17 20:25:22
+# @Last Modified time: 2022-03-11 17:44:21
 
 import os
 from collections import defaultdict
@@ -50,6 +50,7 @@ class Dandelion:
         self.graph = graph
         self.threshold = None
         self.germline = {}
+        self.querier = None
 
         if germline is not None:
             self.germline.update(germline)
@@ -115,6 +116,126 @@ class Dandelion:
         """
         return copy.deepcopy(self)
 
+    def update_plus(
+        self,
+        option: Literal[
+            'all', 'sequence', 'mutations', 'cdr3 lengths',
+            'mutations and cdr3 lengths'] = 'mutations and cdr3 lengths'):
+        """
+        Retrieve additional data columns that are useful.
+        Parameters
+        ----------
+        self : Dandelion
+            `Dandelion` object.
+        option : Literal
+            One of 'all', 'sequence', 'mutations', 'cdr3 lengths',
+            'mutations and cdr3 lengths'
+        Returns
+        -------
+        Udpated metadata.
+        """
+        mutations_type = ['mu_count', 'mu_freq']
+        mutationsdef = [
+            'cdr_r', 'cdr_s', 'fwr_r', 'fwr_s', '1_r', '1_s', '2_r', '2_s',
+            '3_r', '3_s', '4_r', '4_s', '5_r', '5_s', '6_r', '6_s', '7_r',
+            '7_s', '8_r', '8_s', '9_r', '9_s', '10_r', '10_s', '11_r', '11_s',
+            '12_r', '12_s', '13_r', '13_s', '14_r', '14_s', '15_r', '15_s',
+            '16_r', '16_s', '17_r', '17_s', '18_r', '18_s', '19_r', '19_s',
+            '20_r', '20_s', '21_r', '21_s', '22_r', '22_s', '23_r', '23_s',
+            '24_r', '24_s', '25_r', '25_s', '26_r', '26_s', '27_r', '27_s',
+            '28_r', '28_s', '29_r', '29_s', '30_r', '30_s', '31_r', '31_s',
+            '32_r', '32_s', '33_r', '33_s', '34_r', '34_s', '35_r', '35_s',
+            '36_r', '36_s', '37_r', '37_s', '38_r', '38_s', '39_r', '39_s',
+            '40_r', '40_s', '41_r', '41_s', '42_r', '42_s', '43_r', '43_s',
+            '44_r', '44_s', '45_r', '45_s', '46_r', '46_s', '47_r', '47_s',
+            '48_r', '48_s', '49_r', '49_s', '50_r', '50_s', '51_r', '51_s',
+            '52_r', '52_s', '53_r', '53_s', '54_r', '54_s', '55_r', '55_s',
+            '56_r', '56_s', '57_r', '57_s', '58_r', '58_s', '59_r', '59_s',
+            '60_r', '60_s', '61_r', '61_s', '62_r', '62_s', '63_r', '63_s',
+            '64_r', '64_s', '65_r', '65_s', '66_r', '66_s', '67_r', '67_s',
+            '68_r', '68_s', '69_r', '69_s', '70_r', '70_s', '71_r', '71_s',
+            '72_r', '72_s', '73_r', '73_s', '74_r', '74_s', '75_r', '75_s',
+            '76_r', '76_s', '77_r', '77_s', '78_r', '78_s', '79_r', '79_s',
+            '80_r', '80_s', '81_r', '81_s', '82_r', '82_s', '83_r', '83_s',
+            '84_r', '84_s', '85_r', '85_s', '86_r', '86_s', '87_r', '87_s',
+            '88_r', '88_s', '89_r', '89_s', '90_r', '90_s', '91_r', '91_s',
+            '92_r', '92_s', '93_r', '93_s', '94_r', '94_s', '95_r', '95_s',
+            '96_r', '96_s', '97_r', '97_s', '98_r', '98_s', '99_r', '99_s',
+            '100_r', '100_s', '101_r', '101_s', '102_r', '102_s', '103_r',
+            '103_s', '104_r', '104_s', 'cdr1_r', 'cdr1_s', 'cdr2_r', 'cdr2_s',
+            'fwr1_r', 'fwr1_s', 'fwr2_r', 'fwr2_s', 'fwr3_r', 'fwr3_s', 'v_r',
+            'v_s'
+        ]
+        mutations = [] + mutations_type
+        for m in mutations_type:
+            for d in mutationsdef:
+                mutations.append(m + '_' + d)
+
+        vdjlengths = [
+            'junction_length',
+            'junction_aa_length',
+            'np1_length',
+            'np2_length',
+        ]
+        seqinfo = [
+            'sequence', 'sequence_alignment', 'sequence_alignment_aa',
+            'junction', 'junction_aa', 'germline_alignment', 'fwr1', 'fwr1_aa',
+            'fwr2', 'fwr2_aa', 'fwr3', 'fwr3_aa', 'fwr4', 'fwr4_aa', 'cdr1',
+            'cdr1_aa', 'cdr2', 'cdr2_aa', 'cdr3', 'cdr3_aa',
+            'v_sequence_alignment_aa', 'd_sequence_alignment_aa',
+            'j_sequence_alignment_aa'
+        ]
+        mutations = [x for x in mutations if x in self.data]
+        vdjlengths = [x for x in vdjlengths if x in self.data]
+        seqinfo = [x for x in seqinfo if x in self.data]
+
+        if option == 'all':
+            if len(mutations) > 0:
+                update_metadata(self,
+                                retrieve=mutations,
+                                retrieve_mode='split and average')
+                update_metadata(self,
+                                retrieve=mutations,
+                                retrieve_mode='average')
+            if len(vdjlengths) > 0:
+                update_metadata(self,
+                                retrieve=vdjlengths,
+                                retrieve_mode='split and average')
+            if len(seqinfo) > 0:
+                update_metadata(self,
+                                retrieve=seqinfo,
+                                retrieve_mode='split and unique only')
+        if option == 'sequence':
+            if len(seqinfo) > 0:
+                update_metadata(self,
+                                retrieve=seqinfo,
+                                retrieve_mode='split and unique only')
+        if option == 'mutations':
+            if len(mutations) > 0:
+                update_metadata(self,
+                                retrieve=mutations,
+                                retrieve_mode='split and average')
+                update_metadata(self,
+                                retrieve=mutations,
+                                retrieve_mode='average')
+        if option == 'cdr3 lengths':
+            if len(vdjlengths) > 0:
+                update_metadata(self,
+                                retrieve=vdjlengths,
+                                retrieve_mode='split and average')
+        if option == 'mutations and cdr3 lengths':
+            if len(mutations) > 0:
+                update_metadata(self,
+                                retrieve=mutations,
+                                retrieve_mode='split and average')
+                update_metadata(self,
+                                retrieve=mutations,
+                                retrieve_mode='average')
+            if len(vdjlengths) > 0:
+                update_metadata(self,
+                                retrieve=vdjlengths,
+                                retrieve_mode='split and average')
+
     def update_germline(self,
                         corrected: Optional[Union[Dict, str]] = None,
                         germline: Optional[str] = None,
@@ -143,9 +264,11 @@ class Dandelion:
             try:
                 gml = env['GERMLINE']
             except:
-                raise OSError(
-                    'Environmental variable GERMLINE must be set. Otherwise, please provide path to folder containing germline IGHV, IGHD, and IGHJ fasta files.'
-                )
+                raise KeyError((
+                    'Environmental variable GERMLINE must be set. Otherwise, '
+                    +
+                    'please provide path to folder containing germline IGHV, IGHD, and IGHJ fasta files.'
+                ))
             gml = gml + 'imgt/' + org + '/vdj/'
         else:
             if os.path.isdir(germline):
@@ -153,29 +276,39 @@ class Dandelion:
             elif type(germline) is not list:
                 germline_ = [germline]
                 if len(germline_) < 3:
-                    raise OSError(
-                        'Input for germline is incorrect. Please provide path to folder containing germline IGHV, IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ fasta files (with .fasta extension) as a list.'
-                    )
+                    raise TypeError((
+                        'Input for germline is incorrect. Please provide path to folder containing germline IGHV, '
+                        +
+                        'IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ '
+                        + 'fasta files (with .fasta extension) as a list.'))
                 else:
                     gml = []
                     for x in germline_:
                         if not x.endswith('.fasta'):
-                            raise OSError(
-                                'Input for germline is incorrect. Please provide path to folder containing germline IGHV, IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ fasta files (with .fasta extension) as a list.'
-                            )
+                            raise TypeError((
+                                'Input for germline is incorrect. Please provide path to folder containing germline '
+                                +
+                                'IGHV, IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, '
+                                +
+                                'and IGHJ fasta files (with .fasta extension) as a list.'
+                            ))
                         gml.append(x)
             elif type(germline) is list:
                 if len(germline) < 3:
-                    raise OSError(
-                        'Input for germline is incorrect. Please provide path to folder containing germline IGHV, IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ fasta files (with .fasta extension) as a list.'
-                    )
+                    raise TypeError((
+                        'Input for germline is incorrect. Please provide path to folder containing germline IGHV, IGHD, '
+                        +
+                        'and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ fasta '
+                        + 'files (with .fasta extension) as a list.'))
                 else:
                     gml = []
                     for x in germline:
                         if not x.endswith('.fasta'):
-                            raise OSError(
-                                'Input for germline is incorrect. Please provide path to folder containing germline IGHV, IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ fasta files (with .fasta extension) as a list.'
-                            )
+                            raise TypeError((
+                                'Input for germline is incorrect. Please provide path to folder containing germline '
+                                +
+                                'IGHV, IGHD, and IGHJ fasta files, or individual paths to the germline IGHV, IGHD, and IGHJ fasta '
+                                + 'files (with .fasta extension) as a list.'))
                         gml.append(x)
 
         if type(gml) is not list:
@@ -191,9 +324,9 @@ class Dandelion:
             if 'personalized_ref_dict' in locals():
                 germline_ref.update(personalized_ref_dict)
             else:
-                raise OSError(
-                    'Input for corrected germline fasta is incorrect. Please provide path to file containing corrected germline fasta sequences.'
-                )
+                raise TypeError((
+                    'Input for corrected germline fasta is incorrect. Please provide path to file containing '
+                    + 'corrected germline fasta sequences.'))
 
         self.germline.update(germline_ref)
         logg.info(' finished',
@@ -242,15 +375,12 @@ class Dandelion:
         **kwargs
             passed to `_pickle`.
         """
-        from airr import create_rearrangement
-        writer = create_rearrangement(filename,
-                                      fields=[x for x in self.data],
-                                      **kwargs)
-        for _, row in self.data.iterrows():
-            writer.write(row)
+        data = sanitize_data(self.data)
+        data.to_csv(filename, sep='\t', index=False, **kwargs)
 
     def write_h5(self,
-                 filename: str = 'dandelion_data.h5',
+                 filename: str = 'dandelion_data.h5ddl',
+                 keep_distance: bool = False,
                  complib: Literal['zlib', 'lzo', 'bzip2', 'blosc',
                                   'blosc:blosclz', 'blosc:lz4', 'blosc:lz4hc',
                                   'blosc:snappy', 'blosc:zlib',
@@ -269,7 +399,8 @@ class Dandelion:
         filename
             path to `.h5` file.
         complib : str, Optional
-            method for compression for data frames. see (https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_hdf.html) for more options.
+            method for compression for data frames. see
+            https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_hdf.html
         compression : str, Optional
             same call as complib. Just a convenience option.
         compression_opts : {0-9}, Optional
@@ -347,17 +478,18 @@ class Dandelion:
         except:
             pass
 
-        try:
-            for d in self.distance:
-                # how to make this faster?
-                dat = pd.DataFrame(self.distance[d].toarray())
-                dat.to_hdf(filename,
-                           "distance/" + d,
-                           complib=comp,
-                           complevel=compression_level,
-                           **kwargs)
-        except:
-            pass
+        if keep_distance:
+            try:
+                for d in self.distance:
+                    # how to make this faster?
+                    dat = pd.DataFrame(self.distance[d].toarray())
+                    dat.to_hdf(filename,
+                               "distance/" + d,
+                               complib=comp,
+                               complevel=compression_level,
+                               **kwargs)
+            except:
+                pass
 
         with h5py.File(filename, "a") as hf:
             try:
@@ -385,6 +517,8 @@ class Dandelion:
                 tr = self.threshold
                 hf.create_dataset('threshold', data=tr)
 
+    write = write_h5ddl = write_h5  # shortcut
+
 
 class Query:
     def __init__(self, data):
@@ -485,7 +619,9 @@ class Query:
                     cols.update({query: np.nan})
             ret.update({cell: cols})
         out = pd.DataFrame.from_dict(ret, orient='index')
-        if retrieve_mode not in ['split and sum', 'split and average', 'sum', 'average']:
+        if retrieve_mode not in [
+                'split and sum', 'split and average', 'sum', 'average'
+        ]:
             if retrieve_mode == 'split':
                 for x in out:
                     try:
@@ -520,7 +656,11 @@ def initialize_metadata(self, cols: Sequence, clonekey: str,
                 'retrieve_mode': 'merge and unique only',
             }
         })
-    querier = Query(self.data)
+    if self.querier is None:
+        querier = Query(self.data)
+        self.querier = querier
+    else:
+        querier = self.querier
 
     meta_ = defaultdict(dict)
     for k, v in init_dict.copy().items():
@@ -594,38 +734,60 @@ def initialize_metadata(self, cols: Sequence, clonekey: str,
                 if 'locus' + suffix_l in tmp_metadata:
                     if not check_missing(tmp_metadata.loc[i,
                                                           'locus' + suffix_l]):
-                        tmp_metadata.at[i, 'status'] = tmp_metadata.loc[
+                        tmp_metadata.at[i, 'locus_status'] = tmp_metadata.loc[
                             i, 'locus' +
                             suffix_h] + ' + ' + tmp_metadata.loc[i, 'locus' +
                                                                  suffix_l]
                     else:
-                        tmp_metadata.at[i, 'status'] = tmp_metadata.loc[
-                            i, 'locus' + suffix_h] + '_only'
+                        if '|' in tmp_metadata.at[i, 'locus' + suffix_h]:
+                            tmp_metadata.at[i, 'locus_status'] = 'Multi'
+                        else:
+                            tmp_metadata.at[i,
+                                            'locus_status'] = tmp_metadata.loc[
+                                                i,
+                                                'locus' + suffix_h] + '_only'
                 else:
-                    tmp_metadata.at[i, 'status'] = tmp_metadata.loc[
+                    tmp_metadata.at[i, 'locus_status'] = tmp_metadata.loc[
                         i, 'locus' + suffix_h] + '_only'
             else:
                 if 'locus' + suffix_l in tmp_metadata:
                     if not check_missing(tmp_metadata.loc[i,
                                                           'locus' + suffix_l]):
-                        tmp_metadata.at[i, 'status'] = tmp_metadata.loc[
-                            i, 'locus' + suffix_l] + '_only'
+                        if '|' in tmp_metadata.at[i, 'locus' + suffix_l]:
+                            tmp_metadata.at[i, 'locus_status'] = 'Multi'
+                        else:
+                            tmp_metadata.at[i,
+                                            'locus_status'] = tmp_metadata.loc[
+                                                i,
+                                                'locus' + suffix_l] + '_only'
                     else:
-                        tmp_metadata.at[i, 'status'] = 'unassigned'
+                        tmp_metadata.at[i, 'locus_status'] = 'unassigned'
                 else:
-                    tmp_metadata.at[i, 'status'] = 'unassigned'
+                    tmp_metadata.at[i, 'locus_status'] = 'unassigned'
         else:
             if 'locus' + suffix_l in tmp_metadata:
                 if not check_missing(tmp_metadata.loc[i, 'locus' + suffix_l]):
-                    tmp_metadata.at[i, 'status'] = tmp_metadata.loc[
+                    tmp_metadata.at[i, 'locus_status'] = tmp_metadata.loc[
                         i, 'locus' + suffix_l] + '_only'
                 else:
-                    tmp_metadata.at[i, 'status'] = 'unassigned'
+                    tmp_metadata.at[i, 'locus_status'] = 'unassigned'
             else:
-                tmp_metadata.at[i, 'status'] = 'unassigned'
+                tmp_metadata.at[i, 'locus_status'] = 'unassigned'
 
-    tmp_metadata['status_summary'] = [
-        'Multi' if '|' in i else i for i in tmp_metadata['status']
+    tmp_metadata['locus_status_summary'] = [
+        'Multi' if '|' in i else i for i in tmp_metadata['locus_status']
+    ]
+    acceptable = [
+        'TRB + TRA', 'TRD + TRG', 'IGH + IGK', 'IGH + IGL', 'IGH_only',
+        'TRB_only', 'TRD_only', 'TRA_only', 'TRG_only', 'IGK_only', 'IGL_only',
+        'Multi', 'unassigned'
+    ]
+    tmp_metadata['locus_status'] = [
+        'Multi' if i not in acceptable else i
+        for i in tmp_metadata['locus_status']
+    ]
+    tmp_metadata['locus_status_summary'] = [
+        'Multi' if i == 'Multi' else i for i in tmp_metadata['locus_status']
     ]
 
     for i in tmp_metadata.index:
@@ -960,6 +1122,8 @@ def update_metadata(self: Dandelion,
                 raise ValueError(
                     "Unable to initialize metadata due to missing keys. Please ensure either 'umi_count' or 'duplicate_count' is in the input data."
                 )
+    if 'cell_id' not in self.data:  # shortcut for bulk data to pretend every unique sequence is a cell?
+        self.data['cell_id'] = self.data['sequence_id']
 
     if not all([c in self.data for c in cols]):
         raise ValueError(
@@ -987,7 +1151,11 @@ def update_metadata(self: Dandelion,
     tmp_metadata = self.metadata.copy()
 
     if retrieve is not None:
-        querier = Query(self.data)
+        if self.querier is None:
+            querier = Query(self.data)
+            self.querier = querier
+        else:
+            querier = self.querier
         ret_dict = {}
         if type(retrieve) is str:
             retrieve = [retrieve]
@@ -1011,10 +1179,10 @@ def update_metadata(self: Dandelion,
                 raise KeyError(
                     'Cannot retrieve \'%s\' : Unknown column name.' % k)
         ret_metadata = pd.concat(retrieve_.values(), axis=1, join="inner")
-        ret_metadata.dropna(axis = 1, how = 'all', inplace = True)
+        ret_metadata.dropna(axis=1, how='all', inplace=True)
         for col in ret_metadata:
             if all_missing(ret_metadata[col]):
-                ret_metadata.drop(col, axis = 1, inplace = True)
+                ret_metadata.drop(col, axis=1, inplace=True)
 
         if collapse_alleles:
             for k in ret_dict.keys():
@@ -1035,36 +1203,3 @@ def update_metadata(self: Dandelion,
         for r in ret_metadata:
             tmp_metadata[r] = pd.Series(ret_metadata[r])
         self.metadata = tmp_metadata.copy()
-
-
-def load_data(obj: Union[pd.DataFrame, str]) -> pd.DataFrame:
-    """
-    Reads in or copy dataframe object and set sequence_id as index without dropping.
-
-    Parameters
-    ----------
-    obj : DataFrame, str
-        file path to .tsv file or pandas DataFrame object.
-
-    Returns
-    -------
-    pandas DataFrame object.
-    """
-    if os.path.isfile(str(obj)):
-        try:
-            obj_ = pd.read_csv(obj, sep='\t')
-        except FileNotFoundError as e:
-            print(e)
-    elif isinstance(obj, pd.DataFrame):
-        obj_ = obj.copy()
-    else:
-        raise TypeError(
-            "Either input is not of <class 'pandas.core.frame.DataFrame'> or file does not exist."
-        )
-
-    if 'sequence_id' in obj_.columns:
-        obj_.set_index('sequence_id', drop=False, inplace=True)
-    else:
-        raise KeyError("'sequence_id' not found in columns of input")
-
-    return (obj_)
