@@ -35,8 +35,10 @@ def test_reannotategenes(create_testfolder, database_paths_mouse):
     ddl.pp.reannotate_genes(str(create_testfolder),
                             igblast_db=database_paths_mouse['igblast_db'],
                             germline=database_paths_mouse['germline'],
-                            org='mouse')
-    assert len(list((create_testfolder / 'dandelion/tmp').iterdir())) == 3
+                            org='mouse',
+                            reassign_dj = False,
+                            )
+    assert len(list((create_testfolder / 'dandelion/tmp').iterdir())) == 4
 
 
 def test_reassignalleles(create_testfolder, database_paths_mouse):
@@ -46,7 +48,7 @@ def test_reassignalleles(create_testfolder, database_paths_mouse):
                             org='mouse',
                             novel=True,
                             plot=False)
-    assert len(list((create_testfolder / 'dandelion/tmp').iterdir())) == 6
+    assert len(list((create_testfolder / 'dandelion/tmp').iterdir())) == 7
 
 
 def test_updateblastdb(database_paths_mouse):
@@ -60,7 +62,7 @@ def test_assignsisotypes(create_testfolder, database_paths_mouse,
                            blastdb=database_paths_mouse['blastdb_fasta'],
                            correction_dict=balbc_ighg_primers,
                            plot=False)
-    assert len(list((create_testfolder / 'dandelion').iterdir())) == 4
+    assert len(list((create_testfolder / 'dandelion').iterdir())) == 2
 
 
 def test_create_germlines(create_testfolder, processed_files,
@@ -77,14 +79,14 @@ def test_filtercontigs(create_testfolder, processed_files, dummy_adata_mouse):
     f = create_testfolder / str('dandelion/' + processed_files['filtered'])
     dat = pd.read_csv(f, sep='\t')
     vdj, adata = ddl.pp.filter_contigs(dat, dummy_adata_mouse)
-    assert dat.shape[0] == 1287
-    assert vdj.data.shape[0] == 787
-    assert vdj.metadata.shape[0] == 395
-    assert adata.n_obs == 547
     f1 = create_testfolder / "test.h5"
     f2 = create_testfolder / "test.h5ad"
     vdj.write_h5(f1)
     adata.write_h5ad(f2)
+    assert dat.shape[0] == 1285
+    assert vdj.data.shape[0] == 782
+    assert vdj.metadata.shape[0] == 392
+    assert adata.n_obs == 547
 
 
 def test_generate_network(create_testfolder):
@@ -94,6 +96,17 @@ def test_generate_network(create_testfolder):
     ddl.tl.generate_network(vdj)
     assert vdj.distance is not None
     assert vdj.edges is not None
-    assert vdj.n_obs == 395
+    assert vdj.n_obs == 392
     assert vdj.layout is not None
     assert vdj.graph is not None
+
+
+def test_filtercontigs_drop_contigs(create_testfolder, processed_files, dummy_adata_mouse):
+    f = create_testfolder / str('dandelion/' + processed_files['filtered'])
+    dat = pd.read_csv(f, sep='\t')
+    vdj, adata = ddl.pp.filter_contigs(dat, dummy_adata_mouse,
+                                       filter_poorqualitycontig = True)
+    assert dat.shape[0] == 1285
+    assert vdj.data.shape[0] == 781
+    assert vdj.metadata.shape[0] == 391
+    assert adata.n_obs == 547
