@@ -1,10 +1,13 @@
 #!/usr/bin/env python
 import pandas as pd
 import dandelion as ddl
+import sys
+import pytest
 
 from fixtures import (airr_reannotated, create_testfolder, database_paths)
 
 
+@pytest.mark.skipif(sys.platform == 'darwin', reason="macos CI stalls.")
 def test_importrpy2():
 
     from rpy2.robjects.packages import importr
@@ -15,6 +18,7 @@ def test_importrpy2():
     assert sh.__module__ == 'rpy2.robjects.packages'
 
 
+@pytest.mark.skipif(sys.platform == 'darwin', reason="macos CI stalls.")
 def test_mutation(create_testfolder, airr_reannotated):
     f = create_testfolder / "test.tsv"
     airr_reannotated.to_csv(f, sep='\t', index=False)
@@ -26,6 +30,7 @@ def test_mutation(create_testfolder, airr_reannotated):
     assert not vdj.data.mu_freq.empty
 
 
+@pytest.mark.skipif(sys.platform == 'darwin', reason="macos CI stalls.")
 def test_create_germlines(create_testfolder, database_paths):
     f = create_testfolder / "test.tsv"
     out = pd.read_csv(f, sep='\t')
@@ -34,6 +39,7 @@ def test_create_germlines(create_testfolder, database_paths):
     assert not vdj.data.germline_alignment_d_mask.empty
 
 
+@pytest.mark.skipif(sys.platform == 'darwin', reason="macos CI stalls.")
 def test_manual_threshold_and_define_clones(create_testfolder):
     f = create_testfolder / "test.tsv"
     out = pd.read_csv(f, sep='\t')
