@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+import sys
 import pytest
 import json
 import pandas as pd
@@ -10,6 +11,7 @@ from fixtures import (airr_reannotated, dummy_adata, create_testfolder,
                       json_10x_cr6, dummy_adata_cr6)
 
 
+@pytest.mark.slow
 def test_setup(create_testfolder, airr_reannotated, dummy_adata):
     vdj, adata = ddl.pp.filter_contigs(airr_reannotated, dummy_adata)
     assert airr_reannotated.shape[0] == 9
@@ -23,6 +25,7 @@ def test_setup(create_testfolder, airr_reannotated, dummy_adata):
     assert vdj2.metadata is not None
 
 
+@pytest.mark.slow
 def test_find_clones(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
@@ -33,6 +36,7 @@ def test_find_clones(create_testfolder):
     vdj.write_h5(f)
 
 
+@pytest.mark.slow
 def test_clone_size(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
@@ -42,6 +46,7 @@ def test_clone_size(create_testfolder):
     assert not vdj.metadata.clone_id_size.empty
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize(
     "resample,expected",
     [pytest.param(None, 4), pytest.param(3, 3)])
@@ -64,6 +69,7 @@ def test_generate_network(create_testfolder, resample, expected):
     assert vdj.edges is not None
 
 
+@pytest.mark.slow
 def test_find_clones_key(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
@@ -77,6 +83,7 @@ def test_find_clones_key(create_testfolder):
     assert vdj.graph is not None
 
 
+@pytest.mark.slow
 def test_transfer(create_testfolder, dummy_adata):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
@@ -90,6 +97,7 @@ def test_transfer(create_testfolder, dummy_adata):
     dummy_adata.write_h5ad(f2)
 
 
+@pytest.mark.slow
 def test_diversity_gini(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
@@ -100,19 +108,25 @@ def test_diversity_gini(create_testfolder):
     ddl.tl.clone_diversity(vdj, groupby='sample_id', metric='clone_centrality')
     assert not vdj.metadata.clone_centrality_gini.empty
     assert not vdj.metadata.clone_size_gini.empty
+    tmp = ddl.tl.clone_diversity(vdj,
+                                 groupby='sample_id',
+                                 metric='clone_centrality',
+                                 update_obs_meta=False)
+    assert isinstance(tmp, pd.DataFrame)
 
 
+@pytest.mark.slow
 def test_diversity_gini(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
     ddl.tl.clone_diversity(vdj, groupby='sample_id')
+    tmp = ddl.tl.clone_diversity(vdj,
+                                 groupby='sample_id',
+                                 update_obs_meta=False)
+    assert isinstance(tmp, pd.DataFrame)
 
 
-def test_diversity_gini_simple(create_testfolder):
-    f = create_testfolder / "test.h5"
-    vdj = ddl.read_h5(f)
-
-
+@pytest.mark.slow
 @pytest.mark.parametrize("resample", [True, False])
 def test_diversity_chao(create_testfolder, resample):
     f = create_testfolder / "test.h5"
@@ -129,8 +143,14 @@ def test_diversity_chao(create_testfolder, resample):
                                method='chao1',
                                resample=resample)
     assert not vdj.metadata.clone_size_chao1.empty
+    tmp = ddl.tl.clone_diversity(vdj,
+                                 groupby='sample_id',
+                                 method='chao1',
+                                 update_obs_meta=False)
+    assert isinstance(tmp, pd.DataFrame)
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize("method,diversitykey", [
     pytest.param('chao1', None),
     pytest.param('chao1', 'test_diversity_key'),
@@ -150,6 +170,7 @@ def test_diversity_anndata(create_testfolder, method, diversitykey):
         assert 'test_diversity_key' in adata.uns
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize("resample,normalize", [
     pytest.param(True, True),
     pytest.param(False, True),
@@ -176,8 +197,14 @@ def test_diversity_shannon(create_testfolder, resample, normalize):
         assert not vdj.metadata.clone_size_normalized_shannon.empty
     else:
         assert not vdj.metadata.clone_size_shannon.empty
+    tmp = ddl.tl.clone_diversity(vdj,
+                                 groupby='sample_id',
+                                 method='shannon',
+                                 update_obs_meta=False)
+    assert isinstance(tmp, pd.DataFrame)
 
 
+@pytest.mark.slow
 def test_setup2(create_testfolder, json_10x_cr6, dummy_adata_cr6):
     json_file = str(create_testfolder) + "/test_all_contig_annotations.json"
     with open(json_file, 'w') as outfile:
@@ -197,6 +224,7 @@ def test_setup2(create_testfolder, json_10x_cr6, dummy_adata_cr6):
     adata.write_h5ad(f2)
 
 
+@pytest.mark.slow
 def test_diversity_rarefaction(create_testfolder):
     f = create_testfolder / "test.h5ad"
     adata = sc.read_h5ad(f)
@@ -210,6 +238,7 @@ def test_diversity_rarefaction(create_testfolder):
     assert p is not None
 
 
+@pytest.mark.slow
 def test_diversity_rarefaction2(create_testfolder):
     f = create_testfolder / "test.h5ad"
     adata = sc.read_h5ad(f)
@@ -222,20 +251,23 @@ def test_diversity_rarefaction2(create_testfolder):
     assert p is not None
 
 
+@pytest.mark.slow
 def test_diversity_rarefaction3(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
     vdj.data['sample_id'] = 'sample_test'
     vdj.data['contig_QC_pass'] = 'True'
-    ddl.update_metadata(vdj,
-                        retrieve=['sample_id', 'contig_QC_pass'],
-                        retrieve_mode=['merge and unique only', 'merge and unique only'])
+    ddl.update_metadata(
+        vdj,
+        retrieve=['sample_id', 'contig_QC_pass'],
+        retrieve_mode=['merge and unique only', 'merge and unique only'])
     df = ddl.tl.clone_rarefaction(vdj, groupby='sample_id')
     assert isinstance(df, dict)
     p = ddl.pl.clone_rarefaction(vdj, color='sample_id')
     assert p is not None
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize(
     "metric", ['clone_network', None, 'clone_degree', 'clone_centrality'])
 def test_diversity_gini2(create_testfolder, metric):
@@ -243,9 +275,10 @@ def test_diversity_gini2(create_testfolder, metric):
     vdj = ddl.read_h5(f)
     vdj.data['sample_id'] = 'sample_test'
     vdj.data['contig_QC_pass'] = 'True'
-    ddl.update_metadata(vdj,
-                        retrieve=['sample_id', 'contig_QC_pass'],
-                        retrieve_mode=['merge and unique only', 'merge and unique only'])
+    ddl.update_metadata(
+        vdj,
+        retrieve=['sample_id', 'contig_QC_pass'],
+        retrieve_mode=['merge and unique only', 'merge and unique only'])
     ddl.tl.clone_diversity(vdj,
                            groupby='sample_id',
                            resample=True,
@@ -265,14 +298,16 @@ def test_diversity_gini2(create_testfolder, metric):
         assert not vdj.metadata.clone_centrality_gini.empty
 
 
+@pytest.mark.slow
 def test_diversity2a(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
     vdj.data['sample_id'] = 'sample_test'
     vdj.data['contig_QC_pass'] = 'True'
-    ddl.update_metadata(vdj,
-                        retrieve=['sample_id', 'contig_QC_pass'],
-                        retrieve_mode=['merge and unique only', 'merge and unique only'])
+    ddl.update_metadata(
+        vdj,
+        retrieve=['sample_id', 'contig_QC_pass'],
+        retrieve_mode=['merge and unique only', 'merge and unique only'])
     ddl.tl.clone_diversity(vdj,
                            groupby='sample_id',
                            reconstruct_network=False,
@@ -281,14 +316,16 @@ def test_diversity2a(create_testfolder):
     assert not vdj.metadata.clone_network_vertex_size_gini.empty
 
 
+@pytest.mark.slow
 def test_diversity2b(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
     vdj.data['sample_id'] = 'sample_test'
     vdj.data['contig_QC_pass'] = 'True'
-    ddl.update_metadata(vdj,
-                        retrieve=['sample_id', 'contig_QC_pass'],
-                        retrieve_mode=['merge and unique only', 'merge and unique only'])
+    ddl.update_metadata(
+        vdj,
+        retrieve=['sample_id', 'contig_QC_pass'],
+        retrieve_mode=['merge and unique only', 'merge and unique only'])
     ddl.tl.clone_diversity(vdj,
                            groupby='sample_id',
                            use_contracted=True,
@@ -297,14 +334,16 @@ def test_diversity2b(create_testfolder):
     assert not vdj.metadata.clone_network_vertex_size_gini.empty
 
 
+@pytest.mark.slow
 def test_diversity2c(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
     vdj.data['sample_id'] = 'sample_test'
     vdj.data['contig_QC_pass'] = 'True'
-    ddl.update_metadata(vdj,
-                        retrieve=['sample_id', 'contig_QC_pass'],
-                        retrieve_mode=['merge and unique only', 'merge and unique only'])
+    ddl.update_metadata(
+        vdj,
+        retrieve=['sample_id', 'contig_QC_pass'],
+        retrieve_mode=['merge and unique only', 'merge and unique only'])
     x = ddl.tl.clone_diversity(vdj,
                                groupby='sample_id',
                                key='sequence',
@@ -312,21 +351,7 @@ def test_diversity2c(create_testfolder):
     assert isinstance(x, pd.DataFrame)
 
 
-def test_overlap1(create_testfolder):
-    f = create_testfolder / "test.h5ad"
-    adata = sc.read_h5ad(f)
-    ddl.tl.clone_overlap(adata, groupby='group3', colorby='group2')
-    assert 'clone_overlap' in adata.uns
-    assert isinstance(adata.uns['clone_overlap'], pd.DataFrame)
-    ddl.pl.clone_overlap(adata, groupby='group3', colorby='group2')
-
-
-def test_overlap2(create_testfolder):
-    f = create_testfolder / "test.h5ad"
-    adata = sc.read_h5ad(f)
-    ddl.pl.clone_overlap(adata, groupby='group3', colorby='group2')
-
-
+@pytest.mark.slow
 def test_extract_edge_weights(create_testfolder):
     f = create_testfolder / "test.h5"
     vdj = ddl.read_h5(f)
@@ -334,3 +359,18 @@ def test_extract_edge_weights(create_testfolder):
     assert x is None
     x = ddl.tl.extract_edge_weights(vdj, expanded_only=True)
     assert x is None
+
+
+@pytest.mark.slow
+@pytest.mark.parametrize("method", [
+    pytest.param('chao1'),
+    pytest.param('shannon'),
+])
+def test_diversity_anndata2(create_testfolder, method):
+    f = create_testfolder / "test.h5ad"
+    adata = sc.read_h5ad(f)
+    tmp = ddl.tl.clone_diversity(adata,
+                                 groupby='sample_id',
+                                 method=method,
+                                 update_obs_meta=False)
+    assert isinstance(tmp, pd.DataFrame)
