@@ -47,23 +47,23 @@ def test_reannotategenes(create_testfolder, database_paths):
                             igblast_db=database_paths['igblast_db'],
                             germline=database_paths['germline'],
                             loci='tr')
-    assert len(list((create_testfolder / 'dandelion/tmp').iterdir())) == 3
-    assert len(list((create_testfolder / 'dandelion').iterdir())) == 4
+    assert len(list((create_testfolder / 'dandelion/tmp').iterdir())) == 9
+    assert len(list((create_testfolder / 'dandelion').iterdir())) == 2
 
 
 def test_loadtravdv_reannotated(create_testfolder):
     vdj = ddl.Dandelion(
         str(create_testfolder) +
-        '/dandelion/filtered_contig_igblast_db-pass.tsv')
-    assert vdj.data.shape[0] == 6
-    assert all([i == 'TRD' for i in vdj.data['locus']])
+        '/dandelion/filtered_contig_dandelion.tsv')
+    assert vdj.data.shape[0] == 23
+    assert len([i for i in vdj.data['locus'] if i == 'TRD']) == 0
 
 
 def test_travdv_filter(create_testfolder, dummy_adata_travdv):
     vdj = ddl.Dandelion(
         str(create_testfolder) +
-        '/dandelion/filtered_contig_igblast_db-pass.tsv')
-    assert vdj.data.shape[0] == 6
-    assert all([i == 'TRD' for i in vdj.data['locus']])
+        '/dandelion/filtered_contig_dandelion.tsv')
+    assert vdj.data.shape[0] == 23
+    assert len([i for i in vdj.data['locus'] if i == 'TRD']) == 0
     vdj2, adata = ddl.pp.filter_contigs(vdj, dummy_adata_travdv)
-    assert vdj2.data.shape[0] == 6
+    assert vdj2.data.shape[0] == 8
